package xyz.johnny.norntalk

import android.content.Context
import android.content.Intent
import android.os.AsyncTask
import android.telephony.PhoneNumberUtils
import android.util.Log
import android.view.View
import android.widget.Toast
import com.google.zxing.integration.android.IntentIntegrator
import kotlinx.android.synthetic.main.activity_compose.*
import xyz.johnny.norntalk.messages.*
import xyz.johnny.norntalk.security.QRCode
import xyz.johnny.norntalk.ui_components.ComposeView
import xyz.johnny.norntalk.ui_components.NornCaptureActivity
import java.io.ByteArrayOutputStream
import java.io.PrintStream
import java.lang.ref.WeakReference
import java.util.*


/**
 * Activité permettant de créer une nouvelle conversation
 */
class ComposeActivity : ComposeView.SendActivity() {

    companion object {

        class SendTask(context: Context, val number: String, val text: String, private val medias: Array<NornMedia>) : AsyncTask<Void, Void, NornMessage?>() {

            val context = WeakReference<Context>(context)

            override fun doInBackground(vararg params: Void): NornMessage? {
                try {
                    // liste des contacts
                    val context = this.context.get()!!
                    val contacts = arrayOf(NornContact.getContact(this.number, context))
                    // créer la conversation
                    val conversation = NornConversation.Companion.getConversation(contacts, context)
                    // créer le message
                    val msg = NornMessage(this.text, conversation, Date(), context)
                    for (media in medias)
                        msg.medias.add(media)

                    // envoyer le message
                    return if (msg.send(context)) msg else null
                } catch (e: Exception) {
                    val stream = ByteArrayOutputStream()
                    val out = PrintStream(stream)
                    e.printStackTrace(out)
                    Log.d(this::class.java.simpleName, stream.toString())
                    return null
                }
            }

            override fun onPostExecute(message: NornMessage?) {
                val context = this.context.get()!!

                if (message == null) {
                    Toast.makeText(context, context.getString(R.string.invalid_number), Toast.LENGTH_SHORT).show()
                    return
                }

                // Transmettre le message aux autres activités
                NornMessageDispatcher.send(message)

                // ouvrir la conversation
                val intent = Intent(context, ConversationActivity::class.java)
                intent.putExtra("conversationId", message.conversation.id)
                context.startActivity(intent)
            }

        }

    }

    override val layout = R.layout.activity_compose

    override fun sendMessage(text: String, medias: Array<NornMedia>) {
        // trouver le destinataire saisi par l'utilisateur
        val dest = number.text.toString()

        // vérifie que le numéro de téléphone est correct
        if (!PhoneNumberUtils.isWellFormedSmsAddress(dest)) {
            Toast.makeText(this, getString(R.string.invalid_number), Toast.LENGTH_SHORT).show()
            return
        }

        // envoyer le message en parallèle
        SendTask(this, dest, text, medias).executeOnExecutor(AsyncTask.THREAD_POOL_EXECUTOR)
    }

    override fun startActivity(intent: Intent) {
        super.startActivity(intent)
        this.finish()
    }

    /**
     * Lancer le scan d'un QR code
     *
     * @param view le bouton sur lequel l'utilisateur a appuyé
     */
    fun QRscan(view: View) {
        val intent = IntentIntegrator(this)
        intent.setDesiredBarcodeFormats(IntentIntegrator.QR_CODE_TYPES)
        intent.setBeepEnabled(false)
        intent.setPrompt(getString(R.string.qr_prompt))
        intent.captureActivity = NornCaptureActivity::class.java
        intent.initiateScan()
    }

    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) {
        // lire le résultat du scan
        val result = IntentIntegrator.parseActivityResult(requestCode, resultCode, data)
        if (result != null) {
            if (result.contents == null) {
                Log.d(this::class.java.simpleName, "Cancelled scan")
            } else {
                Log.d(this::class.java.simpleName, "Scanned: " + result.contents)

                // decoder le contenu du QR code
                val decoder = QRCode.Decoder(result.contents)

                // assigner la clé publique au contact
                val contact = NornContact.getContact(decoder.number, this)
                contact.setPublicKey(decoder.pubkey, this)

                // créer la conversation sécurisée
                val conversation = NornConversation.getConversation(arrayOf(contact), this)
                if (!conversation.secured) {
                    conversation.createSecuredConversation(this, decoder.token + ":", decoder.tempkey)
                    conversation.setSecured(true, this)
                }

                // lancer l'activité de la conversation
                val intent = Intent(this, ConversationActivity::class.java)
                intent.putExtra("conversationId", conversation.id)
                this.startActivity(intent)
            }
        } else {
            super.onActivityResult(requestCode, resultCode, data)
        }
    }

}
