package xyz.johnny.norntalk

import android.content.Context
import android.content.Intent
import android.graphics.Bitmap
import android.os.AsyncTask
import android.support.v7.app.AlertDialog
import android.util.Log
import android.view.Menu
import android.view.MenuItem
import android.view.View
import android.widget.ImageView
import android.widget.LinearLayout
import android.widget.ListView
import android.widget.Toast
import kotlinx.android.synthetic.main.content_main.*
import xyz.johnny.norntalk.database.NornDatabase
import xyz.johnny.norntalk.messages.NornMessage
import xyz.johnny.norntalk.messages.NornMessageDispatcher
import xyz.johnny.norntalk.security.QRCode
import xyz.johnny.norntalk.ui_components.BaseActivity
import xyz.johnny.norntalk.ui_components.NornListAdapter
import java.lang.ref.WeakReference


class MainActivity : BaseActivity() {

    companion object {

        class MessagesTask(activity: MainActivity) : AsyncTask<Context, Void, List<NornMessage>>() {

            val activity = WeakReference<MainActivity>(activity)

            override fun doInBackground(vararg params: Context): List<NornMessage> {
                val activity = this.activity.get()!!
                val db = NornDatabase.getNornDatabase(activity)
                return db.messageDao().getLastMessages().map { NornMessage.fromDbMessage(it, activity) }
            }

            override fun onPostExecute(result: List<NornMessage>) {
                super.onPostExecute(result)
                val activity = this.activity.get()!!
                val container = activity.findViewById<ListView>(R.id.fragment_container)
                container.adapter = NornListAdapter(result.toMutableList(), activity)
            }

        }

        class QRCodeTask(context: BaseActivity, val toast: Toast) : AsyncTask<Void, Void, Bitmap>() {

            val context = WeakReference<BaseActivity>(context)

            override fun doInBackground(vararg params: Void?): Bitmap? {
                return QRCode.generate(this.context.get()!!)
            }

            override fun onPostExecute(qrcode: Bitmap?) {
                super.onPostExecute(qrcode)

                if (qrcode != null) {
                    this.toast.setText(R.string.qrcode_generated)
                    this.toast.show()

                    // création de la pop-up
                    val alertDialog = AlertDialog.Builder(this.context.get()!!).create()
                    alertDialog.setTitle(R.string.action_qrcode)

                    // ajouter le QR code généré
                    val qrcode_group = this.context.get()!!.findViewById<LinearLayout>(R.id.qrcode_group)
                    val qrDialog = alertDialog.layoutInflater.inflate(R.layout.qrcode_layout, qrcode_group)
                    qrDialog.findViewById<ImageView>(R.id.qrcode).setImageBitmap(qrcode)
                    alertDialog.setView(qrDialog)

                    // afficher la pop-up
                    alertDialog.show()
                } else {
                    toast.setText(R.string.qrcode_error)
                    toast.show()
                }
            }

        }

    }

    override val layout = R.layout.activity_main

    override fun initialize() {
        // créer la liste de conversations en parallèle
        Log.d(this::class.java.simpleName, "creating conversation list")
        MessagesTask(this).executeOnExecutor(AsyncTask.THREAD_POOL_EXECUTOR)
    }

    /**
     * Lance l'activité de composition de message
     *
     * @param view Le bouton flottant sur lequel l'utilisateur a appuyé
     */
    fun message(view: View) {
        val intent = Intent(this, ComposeActivity::class.java)
        startActivity(intent)
    }

    override fun onCreateOptionsMenu(menu: Menu): Boolean {
        // Inflate the menu; this adds items to the action bar if it is present.
        menuInflater.inflate(R.menu.menu_main, menu)
        return true
    }

    override fun onOptionsItemSelected(item: MenuItem): Boolean {
        // Handle action bar item clicks here. The action bar will
        // automatically handle clicks on the Home/Up button, so long
        // as you specify a parent activity in AndroidManifest.xml.

        return when (item.itemId) {
            R.id.action_settings -> { params(); true }
            R.id.action_qrcode -> { qrcode(); true }
            else -> super.onOptionsItemSelected(item)
        }
    }

    /**
     * Lance l'écran des paramètres de l'application
     */
    fun params() {
        val intent = Intent(this, SettingsActivity::class.java)
        startActivity(intent)
    }

    /**
     * Montre la pop-up contenant un QR code pouvant être scanné par un autre utilisateur de
     * l'application
     */
    fun qrcode() {
        val toast = Toast.makeText(this, getString(R.string.generating_qrcode), Toast.LENGTH_SHORT)
        toast.show()

        QRCodeTask(this, toast).executeOnExecutor(AsyncTask.THREAD_POOL_EXECUTOR)
    }

    override fun onDestroy() {
        super.onDestroy()

        // désabonner l'adaptateur de conversations au nouveaux message lorsque l'activité est détruite
        val container = fragment_container as ListView
        val adapter = container.adapter as NornListAdapter?

        if (adapter != null)
            NornMessageDispatcher.unsubscribe(adapter)
    }

}
