package xyz.johnny.norntalk

import android.support.v7.app.AlertDialog
import android.view.View
import android.widget.CheckBox
import android.widget.CompoundButton
import android.widget.Switch
import kotlinx.android.synthetic.main.activity_settings.*
import kotlinx.android.synthetic.main.notifalert_layout.*
import xyz.johnny.norntalk.ui_components.DismissableActivity


class SettingsActivity : DismissableActivity(), CompoundButton.OnCheckedChangeListener{

    override val layout = R.layout.activity_settings

    override fun initialize() {
        //ouverture de la popup Notifications lors du clic sur le TextView Notifications
        alertNotif.setOnClickListener { view -> popUpNotif(view) }

        // activer le switch du mot de passe en fonction des paramètres actuels
        val switchPasswd = findViewById<Switch>(R.id.switch1)
        switchPasswd.isChecked = this.preferences.getString(getString(R.string.password_key), "") != getString(R.string.default_password)

        switchPasswd.setOnClickListener {
            if (switchPasswd.isChecked) {
                // ouverture de la popup Mot de passe lors du clic sur le Switch Mot de passe (si switch activé)
                popUpPasswd(switchPasswd)
            } else {
                // suppression du mot de passe sinon
                val preferenceEditor = this.preferences.edit()
                preferenceEditor.putString(getString(R.string.password_key), getString(R.string.default_password))
                preferenceEditor.apply()
            }
        }
    }

    /**
     * Crée la pop-up permettant de gérer les options en relation avec les notifications
     */
    fun popUpNotif(view: View) {
        // Création de la popup
        val alertDialog = AlertDialog.Builder(this).create()
        alertDialog.setTitle(R.string.textChoixNotifs)

        // inclure les CheckBox dans la popup
        val inflater = this.layoutInflater
        val dialogView = inflater.inflate(R.layout.notifalert_layout, notifications_group)
        alertDialog.setView(dialogView)

        // les clés de configurations associées aux options
        val contacts_key = getString(R.string.notifications_contact)
        val content_key = getString(R.string.notifications_content)

        // cocher les CheckBox en fonction des paramètres courants de l'application
        val checkboxContact = dialogView.findViewById<CheckBox>(R.id.checkBoxContact)!!
        checkboxContact.isChecked = this.preferences.getBoolean(contacts_key, true)

        val checkboxMessage = dialogView.findViewById<CheckBox>(R.id.checkBoxMessage)!!
        checkboxMessage.isChecked = this.preferences.getBoolean(content_key, true)

        // Bouton 'OK' et action correspondante
        alertDialog.setButton(AlertDialog.BUTTON_POSITIVE, getString(R.string.OK), { _, _ ->
            //Enregistrer l'état des checkbox
            val preferenceEditor = this.preferences.edit()
            preferenceEditor.putBoolean(contacts_key, checkboxContact.isChecked)
            preferenceEditor.putBoolean(content_key, checkboxMessage.isChecked)
            preferenceEditor.apply()
        })

        // Bouton 'Annuler' et action correspondante
        alertDialog.setButton(AlertDialog.BUTTON_NEGATIVE, getString(R.string.cancel), {
            dialogInterface, _ -> dialogInterface.cancel()
        })

        // Affichage de la popup
        alertDialog.show()
    }

    override fun onCheckedChanged(buttonView: CompoundButton, isChecked: Boolean) {}
}