package xyz.johnny.norntalk.database.dao

import android.arch.persistence.room.*
import xyz.johnny.norntalk.database.entities.Contact

/**
 * Opérations associées à l'entité [Contact]
 */
@Dao
interface ContactDao {

    /**
     * Retourne l'ensemble des contacts dans la base de données
     *
     * @return Ensemble des contacts
     */
    @Query("select * from Contact")
    fun getAllContacts(): List<Contact>

    /**
     * Retourne le contact associé à un numéro de téléphone donné
     *
     * @param number Numéro de téléphone du contact
     */
    @Query("select * from Contact where number = :number")
    fun getContact(number: String): Contact?

    /**
     * Retourne les contacts participants à une conversation
     *
     * @param convId Identifiant de la conversation
     * @return contacts participants à la conversation
     */
    @Query("""select number, pubkey
        from Contact, Member
        where Member.conversationId = :convId
        and Contact.number = Member.contactNumber
        """)
    fun getContactsFromConversation(convId: Int): List<Contact>

    /**
     * Insérer un contact dans la base de données
     *
     * @param contact Contact à insérer
     */
    @Insert
    fun insertContact(contact: Contact)

    /**
     * Mettre à jour un contact existant
     *
     * @param contact Contact à mettre à jour
     */
    @Update
    fun updateContact(contact: Contact)


    /**
     * Supprimer un contact existant
     *
     * @param contact Contact à supprimer
     */
    @Delete
    fun deleteContact(contact: Contact)

}