package xyz.johnny.norntalk.database.dao

import android.arch.persistence.room.*
import xyz.johnny.norntalk.database.entities.Media

/**
 * Opérations associées à l'entité [Media]
 */
@Dao
interface MediaDao {

    /**
     * Retourne l'ensemble des medias dans la base de données
     *
     * @return L'ensemble des medias
     */
    @Query("select * from Media")
    fun getAllMedias(): List<Media>

    /**
     * Insére un media dans la base de données
     *
     * @param media Media à insérer
     */
    @Insert
    fun insertMedia(media: Media)

    /**
     * Mettre à jour un média donné
     *
     * @param media Média à mettre à jour
     */
    @Update
    fun updateMedia(media: Media)

    /**
     * Supprimer un media donné
     *
     * @param media [Media] à supprimer
     */
    @Delete
    fun deleteMedia(media: Media)

    /**
     * Récupére les documents associés à un message
     *
     * @param messageId Identifiant du message
     */
    @Query("select * from Media where messageId = :messageId order by position")
    fun getMediasFromMessage(messageId: Long) : Array<Media>

}