package xyz.johnny.norntalk.database.dao

import android.arch.persistence.room.*
import xyz.johnny.norntalk.database.entities.Member

/**
 * Opérations associées à l'entité [Member].
 */
@Dao
interface MemberDao {

    /**
     * Retourne l'ensemble des membres présents dans la base de données
     *
     * @return Ensemble des membres
     */
    @Query("select * from Member")
    fun getAllMembers(): List<Member>

    /**
     * Retourne l'ensemble des membres d'une conversation
     *
     * @return Ensemble des membres d'une conversation
     */
    @Query("select contactNumber from Member where conversationId = :conversationId")
    fun getConversationMembers(conversationId: Int): Array<String>

    /**
     * Insére un membre dans la base de données
     *
     * @param member Membre à insérer
     */
    @Insert
    fun insertMember(member: Member)

    /**
     * Mettre à jour un membre donné
     *
     * @param member Membre à mettre à jour
     */
    @Update
    fun updateMember(member: Member)

    /**
     * Supprimer un membre donné
     *
     * @param member Membre à supprimer
     */
    @Delete
    fun deleteMember(member: Member)

}