package xyz.johnny.norntalk.database.entities

import android.arch.persistence.room.ColumnInfo
import android.arch.persistence.room.Entity
import android.arch.persistence.room.ForeignKey
import android.arch.persistence.room.PrimaryKey

/**
 * Représente un média
 *
 * @property type MIME type du média
 * @property uri Adresse du document
 * @property messageId Identifiant du message auquel le média est associé
 * @property name nom du document
 * @property position position du document dans le message
 */
@Entity(foreignKeys = arrayOf(ForeignKey(entity= Message::class,
                                         parentColumns = arrayOf("id"),
                                         childColumns = arrayOf("messageId"),
                                         onDelete = ForeignKey.CASCADE)))
data class Media(var type: String,
                 var uri: String,
                 @ColumnInfo(index=true)
                 var messageId: Long,
                 var name: String,
                 val position: Int) {

    /**
     * Identifiant du média
     */
    @PrimaryKey(autoGenerate = true)
    var id: Long = 0
}