package xyz.johnny.norntalk.database.entities

import android.arch.persistence.room.*
import xyz.johnny.norntalk.database.converters.DateConverters
import java.util.*

/**
 * Représente un message
 *
 * @property text Contenu du message
 * @property date Date d'envoi du message
 * @property read Vrai si le message a été lu par l'utilisateur, Faux sinon
 * @property conversationId Identifiant de la conversation
 * @property key Clé de chiffrement associé au message, inutilisable sans la clé privée de l'utilisateur
 * @property contactNumber numéro de téléphone de l'expéditeur, null si envoyé par l'utilisateur
 */
@Entity(foreignKeys = arrayOf(ForeignKey(entity= Conversation::class,
                                         parentColumns = arrayOf("id"),
                                         childColumns = arrayOf("conversationId"),
                                         onDelete = ForeignKey.CASCADE),
                              ForeignKey(entity = Contact::class,
                                         parentColumns = arrayOf("number"),
                                         childColumns = arrayOf("contactNumber"),
                                         onDelete = ForeignKey.CASCADE)))
@TypeConverters(DateConverters::class)
data class Message(var text: String,
                   var date: Date,
                   var read: Boolean,
                   @ColumnInfo(index=true)
                   var conversationId: Int,
                   var key: ByteArray?,
                   @ColumnInfo(index=true)
                   var contactNumber: String?) {

    /**
     * Identifiant du message
     */
    @PrimaryKey(autoGenerate = true)
    var id: Long = 0

}