package xyz.johnny.norntalk.messages

import android.app.IntentService
import android.app.RemoteInput
import android.content.Intent
import java.util.*

/**
 * Service répondant aux actions présente dans les notifications
 */
class NornMessageService : IntentService("NornMessageService") {

    companion object {

        /**
         * Action marquer comme lu
         */
        val READ_ACTION = "READ"
        /**
         * Action réponse rapide
         */
        val REPLY_ACTION = "REPLY"

    }

    /**
     * Répondre à la conversation
     *
     * @param conversation Conversation concernée
     * @param intent La requête envoyée au service
     */
    fun reply(conversation: NornConversation, intent: Intent) {
        val remoteInput = RemoteInput.getResultsFromIntent(intent)

        // lire le texte entré par l'utilisateur
        val text_reply = if (remoteInput == null) "" else remoteInput.getString("text_reply")

        // créer le message correspondant et l'envoyer
        val message = NornMessage(text_reply, conversation, Date(), this)
        message.send(this)

        // ajouter le message à la notification
        conversation.notification.send(message)
    }

    override fun onHandleIntent(intent: Intent) {
        // trouver la conversation concernée par cette action
        val conversation = NornConversation.getConversation(intent.extras.getInt("conversationId"), this)

        // appeler la méthode correspondant à l'action
        when (intent.action) {
            REPLY_ACTION -> reply(conversation, intent)
            READ_ACTION -> conversation.clear(this)
        }

    }

}