package xyz.johnny.norntalk.ui_components

import android.content.Context
import android.view.View
import android.view.ViewGroup
import android.widget.BaseAdapter
import kotlinx.android.synthetic.main.content_main.*
import xyz.johnny.norntalk.messages.NornMessage

/**
 * Classe de base pour les adaptateur de listes de l'application
 *
 * @property messages Liste de messages à afficher
 * @property context Contexte courant
 * @constructor Crée un adaptateur pour afficher les messages donnés
 */
abstract class BaseNornAdapter(val messages: MutableList<NornMessage>, protected val context: Context) : BaseAdapter(){

    /**
     * Fragment à inclure
     */
    protected abstract val fragment: Int

    /**
     * Ajoute un nouveau message à la liste à l'affichage
     *
     * @param message Message à ajouter
     */
    abstract fun addMessage(message: NornMessage)

    /**
     * Supprime les messages données de l'affichage
     *
     * @param messages Messages à supprimer
     */
    abstract fun deleteMessages(messages: Array<NornMessage>)

    /**
     * Identifiant de la conversation
     */
    abstract val conversationId: Int

    override fun getView(position: Int, convertView: View?, parent: ViewGroup?): View {
        val cView: View
        if (convertView == null) {
            val activity = this.context as BaseActivity
            cView = activity.layoutInflater.inflate(this.fragment, activity.fragment_container, false)
        } else {
            cView = convertView
        }

        return cView
    }

}