package xyz.johnny.norntalk.ui_components


import android.app.Activity
import android.content.Context
import android.content.Intent
import android.util.AttributeSet
import android.util.Log
import android.view.View
import android.widget.Toast
import kotlinx.android.synthetic.main.view_compose.*
import xyz.johnny.norntalk.messages.NornMedia


/**
 * Vue composée d'un champ de texte et d'un bouton d'envoi
 */
class ComposeView: View {

    constructor(context: Context) : super(context)
    constructor(context: Context, attrs: AttributeSet) : super(context, attrs)
    constructor(context: Context, attrs: AttributeSet, defStyle: Int) : super(context, attrs, defStyle)

    /**
     * Classe abstraite pour les activités incluant une [ComposeView] pour envoyer le message
     * lorsque l'utilisateur appuie sur le bouton envoyer
     */
    abstract class SendActivity : DismissableActivity() {

        companion object {

            const val PICK_PICTURE_REQUEST = 10

        }

        protected val medias = ArrayList<NornMedia>()

        override fun initialize() {
            super.initialize()
            this.buttonPicture.setOnClickListener { this.pickPicture() }
        }
        /**
         * Envoyer le message et vider le champs de texte
         *
         * @param view Le bouton "Envoyer"
         */
        fun sendMessage(view: View) {
            val text = composeInput.text.toString()
            this.composeInput.text.clear()
            this.sendMessage(text, this.medias.toTypedArray().copyOf())
            this.medias.removeAll { true }
        }

        fun pickPicture() {
            Log.d(this::class.java.simpleName, "start picker")
            val intent = Intent(Intent.ACTION_GET_CONTENT)
            intent.addCategory(Intent.CATEGORY_OPENABLE)
            intent.type = "image/*"
            this.startActivityForResult(intent, PICK_PICTURE_REQUEST)
        }

        override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) {
            if (requestCode == PICK_PICTURE_REQUEST && resultCode == Activity.RESULT_OK && data != null) {
                val uri = data.data
                val mimeType = "image/jpeg"
                this.medias.add(NornMedia(this, mimeType, uri))
                Toast.makeText(this, uri.toString(), Toast.LENGTH_LONG).show()
            }

            super.onActivityResult(requestCode, resultCode, data)
        }

        /**
         * Envoyer le message [text] avec les documents [medias] au destinataire souhaité
         *
         * @param text Contenu du message
         * @param medias Documents à envoyer
         */
        abstract fun sendMessage(text: String, medias: Array<NornMedia>)

    }

}
