package xyz.johnny.norntalk.ui_components

import android.content.Context
import android.content.Intent
import android.graphics.Typeface
import android.support.constraint.ConstraintLayout
import android.util.AttributeSet
import kotlinx.android.synthetic.main.conversation_view.view.*
import xyz.johnny.norntalk.ConversationActivity
import xyz.johnny.norntalk.messages.NornMessage


/**
 * Vue pour faciliter l'affichage d'une conversation
 */
class ConversationView : ConstraintLayout {

    constructor(context: Context) : super(context)
    constructor(context: Context, attrs: AttributeSet) : super(context, attrs)
    constructor(context: Context, attrs: AttributeSet, defStyle: Int) : super(context, attrs, defStyle)

    private var _message: NornMessage? = null
    /**
     * Le message affiché dans la vue
     */
    val message get() = _message!!

    init {
        // ouvrir la conversation lorsque l'on appuie sur la conversation
        this.setOnClickListener {
            val intent = Intent(context, ConversationActivity::class.java)
            intent.putExtra("conversationId", message.conversation.id)
            context.startActivity(intent)
        }
    }

    /**
     * Afficher un nouveau message
     *
     * @param msg Le message à afficher
     */
    fun setMessage(msg: NornMessage) {
        this._message = msg

        // assigner le contact au QuickContactBadge
        if (message.contact.uri == null)
            this.quickContactBadge.assignContactFromPhone(message.contact.number, false)
        else
            this.quickContactBadge.assignContactUri(message.contact.uri)

        // ajouter l'avatar
        this.quickContactBadge.setOverlay(null)
        if (message.contact.avatar == null)
            this.quickContactBadge.setImageToDefault()
        else
            this.quickContactBadge.setImageDrawable(message.contact.avatar)

        // afficher le titre de la conversation
        this.title.text = message.conversation.name

        // afficher le message
        this.text.text = message.text
        if (message.read)
            this.text.typeface = Typeface.DEFAULT
        else
            this.text.typeface = Typeface.DEFAULT_BOLD
    }

}
