package xyz.johnny.norntalk.ui_components

import android.content.Context
import android.os.Handler
import android.os.Looper
import android.view.View
import android.view.ViewGroup
import kotlinx.android.synthetic.main.fragment_conversation.view.*
import xyz.johnny.norntalk.R
import xyz.johnny.norntalk.database.NornDatabase
import xyz.johnny.norntalk.messages.NornMessage
import xyz.johnny.norntalk.messages.NornMessageDispatcher

/**
 * Classe permettant d'afficher les conversation sur l'écran principal
 *
 * @param messages Derniers messages de chaque conversation
 * @param context Contexte courant
 * @constructor créer la liste de conversation et l'abonner aux nouveaux messages
 */
class NornListAdapter(messages: MutableList<NornMessage>, context: Context) : BaseNornAdapter(messages, context) {

    companion object {

        /**
         * L'identifiant de la liste de conversations
         */
        val conversationId = -1

    }

    override val fragment = R.layout.fragment_conversation
    override val conversationId = NornListAdapter.conversationId

    init {
        NornMessageDispatcher.subscribe(this)
    }

    /**
     * [Handler] pour mettre à jour l'affichage des conversations
     */
    val update = object : Handler(Looper.getMainLooper()) {
        override fun handleMessage(msg: android.os.Message?) {
            this@NornListAdapter.notifyDataSetChanged()
        }
    }

    override fun addMessage(message: NornMessage) {
        // supprimer la conversation si elle existe déjà
        for (i in this.messages.indices) {
            if (this.messages[i].conversation.id == message.conversation.id) {
                this.messages.removeAt(i)
                break
            }
        }

        // rajouter la conversation
        this.messages.add(0, message)
        this.update.sendEmptyMessage(0)
    }

    /**
     * Insérer le message à la liste de conversation
     *
     * @param message Message à insérer
     */
    fun insert(message: NornMessage) {
        var inserted = false
        var i = -1
        while (!inserted && ++i < this.messages.size) {
            if (this.messages[i].date < message.date) {
                this.messages.add(i, message)
                inserted = true
            }
        }

        if (!inserted)
            this.messages.add(message)
    }

    override fun deleteMessages(messages: Array<NornMessage>) {
        // ensemble des identifiants des messages à supprimer
        val messageIds = messages.map { it.id }.toSet()

        NornDatabase.Helper(this.context).run { db ->
            var i = 0
            while (i < this.messages.size) {
                // supprimer les messages
                if (this.messages[i].id in messageIds) {
                    val message = this.messages[i]
                    this.messages.removeAt(i)

                    // trouver le dernier message de la conversation
                    val last = db.messageDao().getLastMessage(message.conversation.id)

                    if (last != null) {
                        // ajouter le message dans la liste des conversations
                        val nLast = NornMessage.fromDbMessage(last, this.context)
                        this.insert(nLast)
                    }

                } else {
                    i++
                }
            }

            // mettre à jour l'affichage
            this.update.sendEmptyMessage(0)
        }
    }

    /**
     * Supprimer une conversation
     *
     * @param conversationId Identifiant de la conversation
     */
    fun delete(conversationId: Int) {
        var i = 0
        while (i < this.messages.size) {
            if (this.messages[i].conversation.id == conversationId) {
                this.messages.removeAt(i)
                break
            } else {
                i++
            }
        }

        this.update.sendEmptyMessage(0)
    }

    override fun getView(position: Int, convertView: View?, parent: ViewGroup?): View {
        val cView = super.getView(position, convertView, parent)

        // afficher la conversation
        val message = this.getItem(position)
        val conversation = cView.conversation as ConversationView
        conversation.setMessage(message)

        return cView
    }

    override fun getItem(position: Int): NornMessage {
        return this.messages[position]
    }

    override fun getItemId(position: Int): Long {
        return this.messages[position].conversation.id.toLong()
    }

    override fun getCount(): Int {
        return this.messages.size
    }

}