package xyz.johnny.norntalk.ui_components

import android.content.Context
import android.view.View
import android.view.ViewGroup
import kotlinx.android.synthetic.main.fragment_message.view.*
import xyz.johnny.norntalk.R
import xyz.johnny.norntalk.messages.NornConversation
import xyz.johnny.norntalk.messages.NornMessage
import xyz.johnny.norntalk.messages.NornMessageDispatcher

/**
 * Classe permettant d'afficher les messages d'une conversation
 *
 * @param messages Messages de la conversation
 * @param context Contexte courant
 * @constructor Crée les messages de la conversation donnée et l'abonner aux messages de la conversation
 */
class NornMessageAdapter(messages: MutableList<NornMessage>, context: Context, override val conversationId: Int) : BaseNornAdapter(messages, context) {

    override val fragment = R.layout.fragment_message
    /**
     * La conversation associée aux messages
     */
    val conversation get() = NornConversation.getConversation(conversationId, context)

    init {
        NornMessageDispatcher.subscribe(this)
    }

    override fun addMessage(message: NornMessage) {
        this.messages.add(message)
        this.notifyDataSetChanged()
    }

    override fun deleteMessages(messages: Array<NornMessage>) {
        val messageIds = messages.map { it.dbMessage.id }.toSet()

        var i = 0
        while (i < this.messages.size) {
            // supprimer les messages
            if (this.messages[i].id in messageIds)
                this.messages.removeAt(i)
            else
                i++
        }

        this.notifyDataSetChanged()
    }

    override fun getItem(position: Int): NornMessage {
        return this.messages[position]
    }

    override fun getItemId(position: Int): Long {
        return this.getItem(position).date.time
    }

    override fun getCount(): Int {
        return this.messages.size
    }

    override fun getView(position: Int, convertView: View?, parent: ViewGroup?): View {
        val cView = super.getView(position, convertView, parent)

        // afficher le message
        val message = this.getItem(position)
        cView.messageText.setMessage(message)

        return cView
    }

}