package xyz.johnny.norntalk

import org.junit.Assert.*
import org.junit.Test
import xyz.johnny.norntalk.security.ChaCha
import xyz.johnny.norntalk.security.Curve
import java.nio.charset.Charset


/**
 * Example local unit test, which will execute on the development machine (host).
 *
 * See [testing documentation](http://d.android.com/tools/testing).
 */
class CryptographyUnitTest {

    @Test
    fun testCrypto() {
        // generate key
        val key = ChaCha.generateKey()

        // encrypt
        val cipher = ChaCha(key)

        val text = "test".toByteArray()
        println("text: " + text.toString(Charset.defaultCharset()))
        val encrypted = cipher.encrypt(text)

        println("cipher text: " + encrypted.toString(Charset.defaultCharset()))
        assertFalse(encrypted.contentEquals(text))
        // decrypt
        val decrypted = cipher.decrypt(encrypted)

        assertEquals("test", decrypted.toString(Charset.defaultCharset()))
    }

    @Test
    fun testDH() {
        val kp1 = Curve.generateKeyPair()
        val curve1 = Curve(kp1.privateKey, kp1.publicKey)

        val kp2 = Curve.generateKeyPair()
        val curve2 = Curve(kp2.privateKey, kp2.publicKey)

        val chacha1 = curve1.getCipher(curve2.publicKey)
        val chacha2 = curve2.getCipher(curve1.publicKey)

        assertArrayEquals(chacha1.key, chacha2.key)

        val text = "test".toByteArray()
        val ciphertext = chacha1.encrypt(text)
        val text2 = chacha2.decrypt(ciphertext)

        assertArrayEquals(text, text2)
    }

    @Test
    fun testEncryptDifferentKeys() {
        val chacha1 = ChaCha(ChaCha.generateKey())
        val chacha2 = ChaCha(ChaCha.generateKey())

        val text = "test".toByteArray()
        val ciphertext = chacha1.encrypt(text)

        assertFalse(chacha1.decrypt(ciphertext).contentEquals(chacha2.decrypt(ciphertext)))
    }

    @Test
    fun testEncryptDifferentIvs() {
        val key = ChaCha.generateKey()
        val chacha = ChaCha(key)

        val text = "test".toByteArray()
        assertFalse(chacha.encrypt(text).contentEquals(chacha.encrypt(text)))
    }

}
